"""
 Coloring Book nitty gritty..

 The main internals of a colornig book. Each book consists of a color
 selection box and a canvas. In addition are buttons for paging through
 the images in the book, a button for taking a snapshot of the canvas, and
 a box that shows the currently selected color.

 Each canvas is a page from the coloring book composed of image layers. Each
 layer is an area of the coloring book that can be colored.
"""

import glob, message, sys, tempfile, ui
from locals import *

class book:
	def __init__ ( self, color = None ):
		self.pages = None
		self.surface = pygame.Surface ( DISPLAY_SIZE ).convert ()
		self.colors = colors ()
		if not color:
			color = DEFAULT_COLOR
		self.color = color
		self.load ()

	def list_pages ( self ):
		if not self.pages:
			pages = glob.glob ( 'pages/*.py' )
			if pages:
				self.pages = pages
			else:
				sys.exit ()
		return self.pages

	def start_page ( self ):
		self.page = 0
		return self.list_pages ()[0]

	def prev_page ( self ):
		if self.page == 0:
			self.page = len ( self.list_pages () ) - 1
		else:
			self.page = self.page - 1
		return self.list_pages ()[self.page]

	def next_page ( self ):	
		if self.page == len ( self.list_pages () ) - 1:
			self.page = 0
		else:
			self.page = self.page + 1
		return self.list_pages ()[self.page]

	def start_canvas ( self ):
		self.canvas = canvas ( self.start_page () )
		self.init_history ()

	def prev_canvas ( self ):
		self.canvas = canvas ( self.prev_page () )
		self.init_history ()

	def next_canvas ( self ):
		self.canvas = canvas ( self.next_page () )
		self.init_history ()

	def load ( self ):
		self.start_canvas ()
		self.chrome = pygame.Surface ( DISPLAY_SIZE ).convert ()
		self.chrome.fill ( CHROME_COLOR )
		self.chrome = ui.raise_surface ( self.chrome )
		buttons = pygame.image.load ( ARROWS_IMAGE ).convert ()
		buttons.set_colorkey ( CANVAS_MASK )
		self.chrome.blit ( buttons, ( 5, 230 ), ( 0, 0, 10, 18 ) )
		self.chrome.blit ( buttons, ( 624, 230 ), ( 13, 0, 10, 18 ) )
		ss = pygame.image.load ( SNAPSHOT_IMAGE ).convert ()
		ss.set_colorkey ( CANVAS_MASK )
		self.chrome.blit ( ss, ( 550, 438 ) )
		undo = pygame.image.load ( UNDO_IMAGE ).convert ()
		undo.set_colorkey ( CANVAS_MASK )
		self.chrome.blit ( undo, ( 500, 436 ) )
		self.selected_color = pygame.Surface ( ( 30, 30 ) ).convert ()

	def check_click ( self, pos ):
		layer = self.canvas.check_click ( ( pos[0] - 20, pos[1] - 20 ) )
		if layer:
			self.history.append ( [ layer, layer.color ] )
			layer.set_color ( self.color )
			return None
		color = self.colors.check_click ( ( pos[0] - 170, pos[1] - 440 ) )
		if color:
			if color is CANVAS_MASK:
				color[2] = color[2] - 1
			self.color = color
			return None
		if pos[0] < 20:
			self.prev_canvas ()
			return None
		if pos[0] > 619:
			self.next_canvas ()
			return None
		if pos[0] > 545 and pos[0] < 575 and pos[1] > 435 and pos[1] < 465:
			self.snapshot ()
			return None
		if pos[0] > 495 and pos[0] < 535 and pos[1] > 435 and pos[1] < 465:
			try:
				last = self.history.pop ()
				last[0].set_color ( last[1] )
			except IndexError:
				pass
			return None

	def init_history ( self ):
		self.history = []
		for layer in self.canvas.layers:
			self.history.append ( [ layer, layer.color ] )

	def draw ( self ):
		self.surface.blit ( self.chrome, ( 0, 0 ) )
		canvas = self.canvas.draw ()
		canvas = ui.lower_surface ( canvas )
		self.surface.blit ( canvas, ( 19, 19 ) )
		colors = self.colors.draw ()
		colors = ui.lower_surface ( colors )
		self.surface.blit ( colors, ( 169, 439 ) )
		self.selected_color.fill ( self.color )
		self.selected_color = ui.lower_surface ( self.selected_color )
		self.surface.blit ( self.selected_color, ( 124, 434 ) )
		return self.surface

	def snapshot ( self ):
		canvas = self.canvas.draw ()
		fd, filename = tempfile.mkstemp ( '.tga', dir = 'snapshots/' )
		pygame.image.save ( canvas, filename )
		pygame.event.post ( pygame.event.Event ( MESSAGE, message = message.message ( 'Snapshot saved as %s.' % ( filename ) ) ) )

class colors:
	def __init__ ( self ):
		self.load ()

	def load ( self ):
		self.surface = pygame.image.load ( COLORS_IMAGE ).convert ()

	def check_click ( self, pos ):
		try:
			color = self.surface.get_at ( pos )
			return color
		except IndexError:
			return None

	def draw ( self ):
		return self.surface

class layer:
	def __init__ ( self, surface, rect, offset, color = None, name = None ):
		self.surface = surface
		self.rect = rect
		self.offset = offset
		self.size = ( rect[2], rect[3] )
		self.background = pygame.Surface ( self.size ).convert ()
		if not color:
			color = BACKGROUND_COLOR
		self.set_color ( color )
		if not name:
			name = ''
		self.name = name

	def check_click ( self, pos ):
		if pos[0] >= self.offset[0] and pos[1] >= self.offset[1] and pos[0] <= ( self.offset[0] + self.size[0] ) and pos[1] <= ( self.offset[1] + self.size[1] ):
			p = ( pos[0] - self.offset[0] + self.rect[0], pos[1] - self.offset[1] + self.rect[1] )
			try:
				if self.surface.get_at ( p ) == LAYER_MASK:
					return True
			except IndexError:
				pass
		return False

	def set_color ( self, color ):
		self.color = color
		self.background.fill ( color )
		self.surface.set_colorkey ( LAYER_MASK )
		self.background.blit ( self.surface, ( 0, 0 ), self.rect )

	def draw ( self, surface ):
		self.background.set_colorkey ( CANVAS_MASK )
		surface.blit ( self.background, self.offset )

class canvas:
	def __init__ ( self, page ):
		self.page = page
		self.surface = pygame.Surface ( CANVAS_SIZE ).convert ()
		self.background ()
		self.load_layers ()

	def check_click ( self, pos ):
#		Only works in Python 2.3 or higher..
#		for layer in self.layers[::-1]:
		layers = self.layers[:]
		layers.reverse ()
		for layer in layers:
			if layer.check_click ( pos ):
				return layer
		return None

	def background ( self ):
		background = pygame.Surface ( DISPLAY_SIZE ).convert ()
		background.fill ( LAYER_MASK )
		self.layers = [ layer ( background, ( 0, 0, 600, 400 ), ( 0, 0 ), name = 'background' ) ]

	def load_layers ( self ):
		page = {}
		execfile ( self.page, page )
		img = pygame.image.load ( page['image'] ).convert ()
		for m in page['map']:
			self.layers.append ( layer ( img, m[0], m[1] ) )

	def draw ( self ):
		for layer in self.layers:
			layer.draw ( self.surface )
		return self.surface
